local addonName, addon = ...
local addoncore = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceEvent-3.0", "AceTimer-3.0", "AceBucket-3.0")
addon.core = addoncore:NewModule("Core", "AceEvent-3.0", "AceTimer-3.0", "AceBucket-3.0")
addon.LDB = LibStub("LibDataBroker-1.1", true)
addon.icon = addon.LDB and LibStub("LibDBIcon-1.0", true)
local core = addon.core
local qtip = LibStub("LibQTip-1.0")
local FONTEND = FONT_COLOR_CODE_CLOSE
local GOLDFONT = NORMAL_FONT_COLOR_CODE
local YELLOWFONT = LIGHTYELLOW_FONT_COLOR_CODE
local WHITEFONT = HIGHLIGHT_FONT_COLOR_CODE
local GRAYFONT = GRAY_FONT_COLOR_CODE
local REDFONT = RED_FONT_COLOR_CODE
local GREENFONT = GREEN_FONT_COLOR_CODE
local MaxLevel = 60

SLASH_NOTTOXIC1, SLASH_NOTTOXIC2 = '/nott', '/nottoxic'
function SlashCmdList.NOTTOXIC(msg, editbox)
	if msg == "config" then
		if _G.InterfaceOptionsFrame:IsShown() then
			_G.InterfaceOptionsFrame:Hide();
		else
			InterfaceOptionsFrame_OpenToCategory("Not Toxic")
			InterfaceOptionsFrame_OpenToCategory("Not Toxic")
		end
	else if msg == "minimap" then
		addon.db.MinimapIcon.hide = not addon.db.MinimapIcon.hide
		addon.icon:Refresh(addonName)
		redrawoptions()
	else
		addon:ToggleDetached()
	end end
end

addon.defaultDB = {
	DBVersion = 12,
	Alts = {},
	Currency = {
		[1] = {ID=1810,Show=true}, -- Redeemed Soul
		[2] = {ID=1813,Show=true}, -- Anima
		[3] = {ID=1828,Show=true}, -- Soul Ash
		[4] = {ID=1767,Show=true}, -- Stygia
		[5] = {ID=1885,Show=true}, -- Offerings
		[6] = {ID=1816,Show=true}, -- Sinstones
	},
	Keystones = {
		[375] = "MISTS",
		[376] = "WAKE",
		[377] = "DE OS",
		[378] = "HALLS",
		[379] = "PFALL",
		[380] = "DEPTH",
		[381] = "SPIRE",
		[382] = "PAIN",
		},
	Factions = {
		[1] = {ID=2407,Show=true}, -- The Ascended
		[2] = {ID=2410,Show=true}, -- The Undying Army
		[3] = {ID=2465,Show=true}, -- The Wild Hunt
		[4] = {ID=2413,Show=true}, -- Court of Harvesters
		[5] = {ID=2439,Show=true}, -- The Avowed
		[6] = {ID=2432,Show=true}, -- Ve'Nari
		},
	Tooltip = {
		Scale = 1,
		AcceptWeekly = true,
		},
	MinimapIcon = { hide = false },
	}

local tip = nil
local thisAlt = UnitName("player") .. " - " .. GetRealmName()

local RAID_CLASS_COLORS = {
    ["Hunter"] = { r = 0.67, g = 0.83, b = 0.45, colorStr = "ffabd473" },
    ["Warlock"] = { r = 0.58, g = 0.51, b = 0.79, colorStr = "ff9482c9" },
    ["Priest"] = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" },
    ["Paladin"] = { r = 0.96, g = 0.55, b = 0.73, colorStr = "fff58cba" },
    ["Mage"] = { r = 0.41, g = 0.8, b = 0.94, colorStr = "ff69ccf0" },
    ["Rogue"] = { r = 1.0, g = 0.96, b = 0.41, colorStr = "fffff569" },
    ["Druid"] = { r = 1.0, g = 0.49, b = 0.04, colorStr = "ffff7d0a" },
    ["Shaman"] = { r = 0.0, g = 0.44, b = 0.87, colorStr = "ff0070de" },
    ["Warrior"] = { r = 0.78, g = 0.61, b = 0.43, colorStr = "ffc79c6e" },
    ["Death Knight"] = { r = 0.77, g = 0.12 , b = 0.23, colorStr = "ffc41f3b" },
    ["Monk"] = { r = 0.0, g = 1.00 , b = 0.59, colorStr = "ff00ff96" },
    ["Demon Hunter"] = { r = 0.64, g = 0.19 , b = 0.79, colorStr = "ffa330c9" },
}
local REP_COLORS = {
	["Neutral"] = {colorStr = "ffff4400"},
	["Friendly"] = {colorStr = "ffff8800"},
	["Honored"] = {colorStr = "ffffcc00"},
	["Revered"] = {colorStr = "ffcccc00"},
	["Exalted"] = {colorStr = "ffccff00"},
	["Paragon"] = {colorStr = "ff44ff00"},
	["Apprehensive"] = {colorStr = "ffff4400"},
	["Tentative"] = {colorStr = "ffff8800"},
	["Ambivalent"] = {colorStr = "ffffcc00"},
	["Cordial"] = {colorStr = "ffcccc00"},
	["Appreciative"] = {colorStr = "ffccff00"},
}

function addon:SkinThis(frame)
	if IsAddOnLoaded("ElvUI") or IsAddOnLoaded("Tukui") then
		if frame.StripTextures then
		frame:StripTextures()
		end
		if frame.SetTemplate then
		frame:SetTemplate("Transparent")
		end
	end
end

local function ClassColor(class, text)
	local color = RAID_CLASS_COLORS[class].colorStr or nil
	if color then
		color = "|c"..color
	else
		color = GRAYFONT
	end
	return color .. text .. FONTEND
end
local function RepColor(standing, text)
	if standing == "" then return "" end
	local color = REP_COLORS[standing].colorStr or nil
	if color then
		color = "|c"..color
	else
		color = YELLOWFONT
	end
	return color .. text .. FONTEND
end

function GetResetTime()
	local nextdayreset = time() + GetQuestResetTime()
	local datedata = C_DateAndTime.GetCurrentCalendarTime()
	local calday, calhour = datedata["weekday"], datedata["hour"]
	if calhour < 8 then calday = calday - 1 end
	calday = calday - 3
	if calday < 1 then calday = calday + 7 end
	local lastweekreset = nextdayreset - (calday * 86400)
	if lastweekreset > time() then return 0 end
	return lastweekreset
end

function core:CharacterInit()
	local ci = addon.db.Alts[thisAlt] or { }
	addon.db.Alts[thisAlt] = ci
	ci.Class = UnitClass("player")
	ci.Level = UnitLevel("player")
	ci.Faction = UnitFactionGroup("player")
	if ci.Faction == "Neutral" then ci.Faction = "Horde" end -- assume unbound pandas are horde, to solve problems
	ci.Show = ci.Show or "Yes"
	local altorder = 0
	for a in pairs(addon.db.Alts) do
		altorder = altorder + 1
	end
	ci.Order = ci.Order or altorder
	addon:UpdateCovenantData()
end

function addon:UpdateCovenantData()
	local cc = addon.db.Alts[thisAlt]
	if not cc.Covenant then
		cc.Covenant = {}
	end
	if C_Covenants.GetCovenantData(C_Covenants.GetActiveCovenantID()) then
		cc.Covenant = C_Covenants.GetCovenantData(C_Covenants.GetActiveCovenantID())
	else
		cc.Covenant = {ID=0, name="None"}
	end
	
	if not cc.Soulbind then
		cc.Soulbind = {}
	end
	if C_Soulbinds.GetSoulbindData(C_Soulbinds.GetActiveSoulbindID()) and C_Soulbinds.GetActiveSoulbindID() ~= 0 then
		cc.Soulbind = C_Soulbinds.GetSoulbindData(C_Soulbinds.GetActiveSoulbindID())
		if cc.Soulbind.ID == 3 then cc.Soulbind.name = "Draven" end
		if cc.Soulbind.ID == 4 then cc.Soulbind.name = "Marileth" end
		if cc.Soulbind.ID == 8 then cc.Soulbind.name = "Nadjia" end
		if cc.Soulbind.ID == 9 then cc.Soulbind.name = "Theotar" end
		if cc.Soulbind.ID == 10 then cc.Soulbind.name = "Heirmir" end
		if cc.Soulbind.ID == 18 then cc.Soulbind.name = "Mikanikos" end
	else
		cc.Soulbind = {ID=0, name="None"}
	end
	cc.Renown = C_CovenantSanctumUI.GetRenownLevel()
end

function addon:CovenantHoverFrame(aframe, alt)
	if covtip then qtip:Release(covtip); covtip = nil end
	covtip = qtip:Acquire("NotToxicCovenantTooltip", 1, "LEFT")
	covtip:SetCellMarginH(0)
	covtip:SetCellMarginV(0)
	covtip:SmartAnchorTo(aframe)
	covtip:ClearAllPoints()
	covtip:SetPoint("BOTTOMLEFT", aframe, "TOPRIGHT")
	covtip:Clear()
	covtip:SetScale(addon.db.Tooltip.Scale)
	covtip:SetFrameLevel(105)
	
	local rewards = ""
	if C_CovenantSanctumUI.GetRenownRewardsForLevel(alt.Covenant.ID, alt.Renown)[1] then rewards = C_CovenantSanctumUI.GetRenownRewardsForLevel(alt.Covenant.ID, alt.Renown)[1].description end
	if not rewards and C_CovenantSanctumUI.GetRenownRewardsForLevel(alt.Covenant.ID, alt.Renown)[1] then rewards = C_CovenantSanctumUI.GetRenownRewardsForLevel(alt.Covenant.ID, alt.Renown)[1].toastDescription end
	covtip:AddLine()
	covtip:SetCell(1, 1, GOLDFONT.."This Level:"..FONTEND)
	if rewards then
		covtip:AddLine()
		covtip:SetCell(2, 1, GREENFONT..rewards..FONTEND)
	else
		covtip:AddLine()
		covtip:SetCell(2, 1, GREENFONT.."Unknown"..FONTEND)
	end
	if alt.Renown<40 then
		if rewards then rewards = "" end
		if C_CovenantSanctumUI.GetRenownRewardsForLevel(alt.Covenant.ID, (alt.Renown + 1))[1] then rewards = C_CovenantSanctumUI.GetRenownRewardsForLevel(alt.Covenant.ID, (alt.Renown + 1))[1].description end
		if not rewards and C_CovenantSanctumUI.GetRenownRewardsForLevel(alt.Covenant.ID, (alt.Renown + 1))[1] then rewards = C_CovenantSanctumUI.GetRenownRewardsForLevel(alt.Covenant.ID, (alt.Renown + 1))[1].toastDescription end
		covtip:AddLine()
		covtip:SetCell(3, 1, GOLDFONT.."Next Level:"..FONTEND)
		if rewards then
			covtip:AddLine()
			covtip:SetCell(4, 1, GREENFONT..rewards..FONTEND)
		else
			covtip:AddLine()
			covtip:SetCell(4, 1, GREENFONT.."Unknown"..FONTEND)
		end
	end
	
	addon:SkinThis(covtip)
	covtip:Show()
end

function addon:UpdateMythicData()
	local cc = addon.db.Alts[thisAlt]
	if C_MythicPlus.IsMythicPlusActive() then
		local currentkeylevel = C_MythicPlus.GetOwnedKeystoneLevel()
		local currentkeyid = C_MythicPlus.GetOwnedKeystoneChallengeMapID()
		for id, name in pairs(addon.db.Keystones) do
			if id == currentkeyid then currentkeyid = name end
		end
		cc.CurrentKeyID = currentkeyid
		cc.CurrentKeyLevel = currentkeylevel
		cc.MythicRunHistory = C_MythicPlus.GetRunHistory()
	end
end

function addon:MythicHoverFrame(aframe, alt)
	if mythictip then qtip:Release(mythictip); mythictip = nil end
	mythictip = qtip:Acquire("NotToxicMythicTooltip", 2, "RIGHT", "LEFT")
	mythictip:SetCellMarginH(0)
	mythictip:SetCellMarginV(0)
	mythictip:SmartAnchorTo(aframe)
	mythictip:ClearAllPoints()
	mythictip:SetPoint("BOTTOMLEFT", aframe, "TOPRIGHT")
	mythictip:Clear()
	mythictip:SetScale(addon.db.Tooltip.Scale)
	mythictip:SetFrameLevel(105)
	local currentline = 0
	local headerdrawn = false
	for _, MythicRun in pairs(alt.MythicRunHistory) do
		if MythicRun.thisWeek then
			if not headerdrawn then
				mythictip:AddLine()
				currentline = currentline + 1
				mythictip:SetCell(currentline, 1, GOLDFONT.."Mythic"..FONTEND)
				mythictip:SetCell(currentline, 2, GOLDFONT.."History"..FONTEND)
				headerdrawn = true
			end
			mythictip:AddLine()
			currentline = currentline + 1
			mythictip:SetCell(currentline, 1, YELLOWFONT..addon.db.Keystones[(MythicRun.mapChallengeModeID)]..FONTEND)
			if MythicRun.completed then
				mythictip:SetCell(currentline, 2, GREENFONT..MythicRun.level..FONTEND)
			else
				mythictip:SetCell(currentline, 2, REDFONT..MythicRun.level..FONTEND)
			end
		end
	end
	if headerdrawn then
		addon:SkinThis(mythictip)
		mythictip:Show()
	end
end

function addon:UpdateRewardData()
	local cc = addon.db.Alts[thisAlt]
	cc.RaidChest = C_WeeklyRewards.GetActivities(3)
	cc.MythicChest = C_WeeklyRewards.GetActivities(1)
	cc.PvpChest = C_WeeklyRewards.GetActivities(2)
	cc.CollectChest = C_WeeklyRewards.HasAvailableRewards()
end

function addon:UpdateFactionData()
	local cc = addon.db.Alts[thisAlt]
	if not cc.Reputation then cc.Reputation = {} end
	for n, f in pairs(addon.db.Factions) do
		local fName, _, fStand, minRep, maxRep, totalRep = GetFactionInfoByID(f.ID)
		if fName then
			if f.ID == 2432 then
				if fStand==1 then fStand="" end
				if fStand==2 then fStand="Apprehensive" end
				if fStand==3 then fStand="Tentative" end
				if fStand==4 then fStand="Ambivalent" end
				if fStand==5 then fStand="Cordial" end
				if fStand==6 then fStand="Appreciative" end
			else
				if fStand<=3 then fStand="" end
				if fStand==4 then fStand="Neutral" end
				if fStand==5 then fStand="Friendly" end
				if fStand==6 then fStand="Honored" end
				if fStand==7 then fStand="Revered" end
				if fStand==8 then fStand="Exalted" end
			end
			local percentRep = ""
			if C_Reputation.IsFactionParagon(f.ID) then
				local paragonRep, MaxParagonRep,_,reward = C_Reputation.GetFactionParagonInfo(f.ID)
				fStand = "Paragon"
				if reward then
					percentRep = "(C) ".. floor(((paragonRep - (floor(paragonRep/MaxParagonRep) * MaxParagonRep))/ MaxParagonRep) * 100)
				else
					percentRep = floor(((paragonRep - (floor(paragonRep/MaxParagonRep) * MaxParagonRep))/ MaxParagonRep) * 100)
				end
				paragonRep, MaxParagonRep,reward = nil
			else
				if minRep and maxRep and totalRep and minRep ~= 42000 then
					percentRep = floor((totalRep - minRep) / (maxRep - minRep) * 100)
				end
			end
			cc.Reputation[f.ID] = {["Faction"] = fname, ["Standing"] = fStand, ["Percent"] = percentRep }
		end
	end
end

function addon:UpdateCallingData()
	local cc = addon.db.Alts[thisAlt]
	if not cc.Callings and C_CovenantCallings.AreCallingsUnlocked() then
		cc.Callings = {
		[1] = {ID=-1,E=0},
		[2] = {ID=-1,E=0},
		[3] = {ID=-1,E=0},
		}
	end
	for _,alt in pairs(addon.db.Alts) do
		if alt.Callings then
			for day = 1,3 do
				local trueday = floor((alt.Callings[day].E-time())/86400)+1
				if trueday ~= day and alt.Callings[day].ID > 0 then
					if trueday >=1 then
						alt.Callings[trueday] = alt.Callings[day]
						alt.Callings[day] = {ID=0,E=0}
					else
						alt.Callings[day] = {ID=0,E=0}
					end
				end
			end
		end
	end
	if cc.Callings then
		for day = 1,3 do
			cc.Callings[day].C = false
			if cc.Callings[day].ID > 0 and not C_QuestLog.IsOnQuest(cc.Callings[day].ID) then 
				cc.Callings[day].C = true
			end
		end
		
		for qlid = 1,C_QuestLog.GetNumQuestLogEntries() do
			local qid = C_QuestLog.GetQuestIDForLogIndex(qlid)
			if C_QuestLog.IsQuestCalling(qid) and C_TaskQuest.GetQuestTimeLeftSeconds(qid) then
				local day = floor(C_TaskQuest.GetQuestTimeLeftSeconds(qid)/86400)+1
				cc.Callings[day].ID = qid
				cc.Callings[day].E = C_TaskQuest.GetQuestTimeLeftSeconds(qid) + time()
				cc.Callings[day].QL = GetQuestLink(qid)
			end
		end
	end
	
	if cc.WeeklyQuest then
		local _,_,_,minprog,maxprog = GetQuestObjectiveInfo(cc.WeeklyQuest,1,false)
		cc.WeeklyMin = minprog
		cc.WeeklyMax = maxprog
	end
	
	for _, q in pairs{62637, 62638, 62633, 62640, 62631, 62634} do
		if C_QuestLog.IsOnQuest(q) or C_QuestLog.IsQuestFlaggedCompleted(q) then
			cc.WeeklyQuest = q
			local _,_,_,minprog,maxprog = GetQuestObjectiveInfo(q,1,false)
			cc.WeeklyMin = minprog
			cc.WeeklyMax = maxprog
		end
	end
	
	if cc.Level >= MaxLevel then
		cc.WeeklyBoss = true
		for _, qid in pairs{61813, 61814, 61815, 61816} do
			if C_TaskQuest.IsActive(qid) then
				cc.WeeklyBoss = false
			end
		end
	end
end

function addon:UpdateTorghast()
	local cc = addon.db.Alts[thisAlt]
	
	local wid = {
		{2925, 2930}, -- Fracture Chambers
		{2926, 2932}, -- Skoldus Hall
		{2924, 2934}, -- Soulforges
		{2927, 2936}, -- Coldheart Interstitia
		{2928, 2938}, -- Mort'regar
		{2929, 2940}, -- The Upper Reaches
	}
	for _, wing in pairs(wid) do
		if C_UIWidgetManager.GetTextWithStateWidgetVisualizationInfo(wing[1]).shownState == 1 then
			local twing = C_UIWidgetManager.GetTextWithStateWidgetVisualizationInfo(wing[1]).text
			twing = strmatch(twing, '|n|cffffffff+(.+)|r')
			local tlevel = C_UIWidgetManager.GetTextWithStateWidgetVisualizationInfo(wing[2]).text
			tlevel = strmatch(tlevel, '|cFF00FF00.+(%d+).+|r')
			if tonumber(tlevel) and tonumber(tlevel) > 0 then
				if not cc.Torghast then cc.Torghast = {} end
				cc.Torghast[twing] = tlevel
			end
		end
	end
end

function addon:UpdateCurrency()
	local cc = addon.db.Alts[thisAlt]
	if not cc.Currency then cc.Currency = {} end
	for id, currency in ipairs(addon.db.Currency) do
		local ctitle = C_CurrencyInfo.GetCurrencyInfo(currency.ID)["name"]
		local icon = C_CurrencyInfo.GetCurrencyInfo(currency.ID)["iconFileID"]
		local amount = C_CurrencyInfo.GetCurrencyInfo(currency.ID)["quantity"]
		if ctitle ~= "" then
			cc.Currency[currency.ID] = {}
			cc.Currency[currency.ID]["Total"] = amount
			cc.Currency[currency.ID]["Icon"] = icon
		end
	end
end

function addon:UpdateItems()
	local cc = addon.db.Alts[thisAlt]
	local baganima = 0
	for bag = 0, 4 do      
		local size = GetContainerNumSlots(bag)
		for pos = 1, size do
			local _,stack,_,quality = GetContainerItemInfo(bag,pos)
			if stack then
				local id = GetContainerItemID(bag,pos);
				local isAnima = C_Item.IsAnimaItemByID(id)
				if isAnima then
					if (quality == 2) and (id ~= 183727) then -- green, but not Resonance of Conflict
						baganima = baganima + 5 * stack
					end
					if quality == 3 then -- blue
						baganima = baganima + 35 * stack
					end
					if quality == 4 then -- purple
						baganima = baganima + 250 * stack
					end
					if id == 183727 then -- Resonance of Conflict
						baganima = baganima + 3 * stack
					end
				end
			end  
		end
	end
	cc.BagAnima = baganima
end

function addon:PickupWeekly()
	if not EncounterJournal then EncounterJournal_LoadUI() end
	ToggleFrame(EncounterJournal)
	--auto-accept weekly bonus quest, if available
	for i = 0, (C_AdventureJournal.GetNumAvailableSuggestions()-1) do
		C_AdventureJournal.SetPrimaryOffset(i)
		local suggest = C_AdventureJournal.GetSuggestions()
		if suggest[1] and suggest[1].title then
			local title = string.sub(suggest[1].title, 1, 5)
			if title == "Bonus" then
				C_AdventureJournal.ActivateEntry(1)
				addon.db.Alts[thisAlt].WeeklyQuestOffer = true
			end
		end
	end
	local suggest = C_AdventureJournal.GetSuggestions()
	if suggest[2] and suggest[2].title then
		local title = string.sub(suggest[2].title, 1, 5)
		if title == "Bonus" then
			C_AdventureJournal.ActivateEntry(2)
			addon.db.Alts[thisAlt].WeeklyQuestOffer = true
		end
	end
	if suggest[3] and suggest[3].title then
		local title = string.sub(suggest[3].title, 1, 5)
		if title == "Bonus" then
			C_AdventureJournal.ActivateEntry(3)
			addon.db.Alts[thisAlt].WeeklyQuestOffer = true
		end
	end
	ToggleFrame(EncounterJournal)
end

function addon:UpdateCharData()
	local cc = addon.db.Alts[thisAlt]
	if not cc then
		core:CharacterInit()
		cc = addon.db.Alts[thisAlt]
	end
	if not cc.LastUpdate then
		cc.LastUpdate = time()
	end
	local ilevel, eilevel = GetAverageItemLevel()
	if ilevel ~= 0 and eilevel ~= 0 then
		cc.ItemLevel = floor(ilevel)
		cc.ItemLevelEquipped = floor(eilevel)
	end
	if not cc.Faction then cc.Faction = UnitFactionGroup("player") end
	
	
	
	for _, alt in pairs(addon.db.Alts) do
		if alt.LastUpdate < GetResetTime() then
			alt.WeeklyQuest = nil
			alt.WeeklyMin = nil
			alt.WeeklyMax = nil
			alt.WeeklyQuestOffer = false
			alt.WeeklyBoss = false
			alt.Torghast = nil
			if alt.RaidChest then
				if not alt.CollectChest then
					for _, i in pairs(alt.RaidChest) do
						if i.level and i.level > 0 then
							alt.CollectChest = true
						end
					end
				end
				alt.RaidChest = nil
			end
			if alt.MythicChest then
				if not alt.CollectChest then
					for _, i in pairs(alt.MythicChest) do
						if i.level and i.level > 0 then
							alt.CollectChest = true
						end
					end
				end
				alt.MythicChest = nil
			end
			if alt.PvpChest then
				if not alt.CollectChest then
					for _, i in pairs(alt.PvpChest) do
						if i.level and i.level > 0 then
							alt.CollectChest = true
						end
					end
				end
				alt.PvpChest = nil
			end
			alt.CurrentKeyID = nil
			alt.CurrentKeyLevel = nil
			alt.LastUpdate = time()
			alt.MythicRunHistory = nil
		end
	end
	
	addon:UpdateTorghast()
	addon:UpdateMythicData()
	addon:UpdateRewardData()
	addon:UpdateFactionData()
	addon:UpdateCurrency()
	addon:UpdateItems()
	
	cc.LastUpdate = time()
end

function addon:showchardata(aframe, alt)
	if minitip then qtip:Release(minitip); minitip = nil end
	minitip = qtip:Acquire("NotToxicMiniTooltip", 2, "RIGHT", "LEFT")
	minitip:SetCellMarginH(0)
	minitip:SetCellMarginV(0)
	minitip:SmartAnchorTo(aframe)
	minitip:ClearAllPoints()
	minitip:SetPoint("BOTTOMLEFT", aframe, "TOPRIGHT")
	minitip:Clear()
	minitip:SetScale(addon.db.Tooltip.Scale)
	minitip:SetFrameLevel(105)

	minitip:AddLine()
	minitip:SetCell(1, 1, GOLDFONT.."Level"..FONTEND)
	minitip:SetCell(1, 2, GREENFONT..alt.Level..FONTEND)
	minitip:AddLine()
	minitip:SetCell(2, 1, GOLDFONT.."Item Level"..FONTEND)
	minitip:SetCell(2, 2, GREENFONT..alt.ItemLevel..FONTEND)
	minitip:AddLine()
	minitip:SetCell(3, 1, GOLDFONT.."Equipped"..FONTEND)
	minitip:SetCell(3, 2, GREENFONT..alt.ItemLevelEquipped..FONTEND)
	
	addon:SkinThis(minitip)
	minitip:Show()
end

function addon:showlink(frame, link)
	GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
	GameTooltip:SetFrameLevel(frame:GetFrameLevel()+1)
	GameTooltip:ClearLines()
	GameTooltip:SetHyperlink(link)
end

function addon:IsDetached()
  return addon.detachframe and addon.detachframe:IsShown()
end
function addon:HideDetached()
  addon.detachframe:Hide()
end
function addon:ToggleDetached()
	if addon:IsDetached() then
    addon:HideDetached()
  else
    addon:ShowDetached()
  end
end
function addon:ShowDetached()
  if not addon.detachframe then
    local f = CreateFrame("Frame","NotToxicDetachFrame",UIParent,BackdropTemplateMixin and "BackdropTemplate,BasicFrameTemplate")
    f:SetMovable(true)
    f:SetFrameStrata("TOOLTIP")
    f:SetFrameLevel(100) -- prevent weird interlacings with other tooltips
    f:SetClampedToScreen(true)
    f:EnableMouse(true)
    f:SetUserPlaced(true)
    f:SetAlpha(1)
    f:SetPoint("CENTER")
    f:SetScript("OnMouseDown", function() f:StartMoving() end)
    f:SetScript("OnMouseUp", function() f:StopMovingOrSizing() end)
    f:SetScript("OnHide", function()
      if tip then
        qtip:Release(tip)
        tip = nil
      end
    end)
    f:SetScript("OnUpdate", function(self)
      if not tip then f:Hide(); return end
      local w,h = tip:GetSize()
      self:SetSize(w*tip:GetScale(),(h+20)*tip:GetScale())
    end)
    f:SetScript("OnKeyDown", function(self,key)
      if key == "ESCAPE" then
        f:SetPropagateKeyboardInput(false)
        f:Hide()
      end
    end)
    f:EnableKeyboard(true)
    addon.detachframe = f
  end
  local f = addon.detachframe
  f:Show()
  addon:SkinThis(f)
  f:SetPropagateKeyboardInput(true)
  if tip then tip:Hide() end
  core:ShowTooltip(f)
end

function core:OnInitialize()
	NotToxicDB = NotToxicDB or addon.defaultDB
	addon.db = NotToxicDB
	
	-- check for database updates and get to latest database model
	
	if NotToxicDB.DBVersion < 8 then --reset DB for shadowlands
		resetDB()
		print("Not Toxic:")
		print("The Database has been cleared and is ready to start Shadowlands, Enjoy.")
	else
		if NotToxicDB.DBVersion < 11 then
			for _,alt in pairs(NotToxicDB.Alts) do
				alt.Torghast = nil
			end
			NotToxicDB.DBVersion = 11
		end
		if NotToxicDB.DBVersion < 12 then
			NotToxicDB.Currency = nil
			NotToxicDB.Currency = addon.defaultDB.Currency
			NotToxicDB.Keystones = nil
			NotToxicDB.Keystones = addon.defaultDB.Keystones
			NotToxicDB.Factions = nil
			NotToxicDB.Factions = addon.defaultDB.Factions
			NotToxicDB.DBVersion = 12
		end
	end
	
	
	core:CharacterInit()
	
	addon.dataobject = addon.LDB and addon.LDB:NewDataObject("NotToxic", {
		text = addonAbbrev,
		type = "launcher",
		icon = "Interface\\Addons\\Not Toxic\\icon.tga",
		OnEnter = function(frame)
			if not addon:IsDetached() then
				core:ShowTooltip(frame)
			end
		end,
		OnLeave = function(frame) end,
		OnClick = function(frame, button)
			if button == "RightButton" then
				if InterfaceOptionsFrame:IsShown() then
					InterfaceOptionsFrame:Hide()
				else
					InterfaceOptionsFrame_OpenToCategory("Not Toxic")
					InterfaceOptionsFrame_OpenToCategory("Not Toxic")
				end
			else if button == "LeftButton" then
				addon:ToggleDetached()
			else
				addon:UpdateCharData()
			end end
			end
		}
	)
	if addon.icon then
		addon.icon:Register(addonName, addon.dataobject, addon.db.MinimapIcon)
		addon.icon:Refresh(addonName)
	end
	core:SetupOptions()
end

local setoptionstable = {}
function core:MakeOptions()
	local optionstable = {
		type = "group",
		name = "Not Toxic",
		icon = "Interface\\Addons\\Not Toxic\\icon.tga",
		args = (function ()
			local ntoptions = {}
			ntoptions["minimapicon"] = {
				order = 1,
				type = "toggle",
				name = "Show Minimap Icon",
				get = function(info)
						return not addon.db.MinimapIcon.hide
					end,
				set = function(info, value)
						addon.db.MinimapIcon.hide = not value
						addon.icon:Refresh(addonName)
						redrawoptions()
					end,
			}
			ntoptions["scale"] = {
				order = 2,
				type = "range",
				name = "Scale",
				desc = "Set scale of tooltip",
				min = 0.5,
				max = 2,
				width = "full",
				set = function(info, value)
					addon.db.Tooltip["Scale"] = value
					end,
				get = function(info, value)
					return addon.db.Tooltip["Scale"]
					end,
			}
			ntoptions["Char"] = {
				order = 3,
				type = "group",
				name = "Characters",
				args = (function()
					local altoptions = {}
					local orderlist = {}
					for _, a in pairs(addon.db.Alts) do
						orderlist[a.Order] = a.Order
					end
					for altname, alt in pairs(addon.db.Alts) do
						altoptions[altname.."_desc"] = {
							order = (alt.Order * 4) - 3,
							name = ClassColor(alt.Class, altname),
							desc = altname,
							descStyle = "tooltip",
							type = "description",
							width = "half",
						}
						altoptions[altname.."_order"] = {
							order = (alt.Order * 4) - 2,
							name = "",
							type = "select",
							width = "half",
							values = orderlist,
							get = function(info) return alt.Order end,
							set = function(info, value)
								for _, a in pairs(addon.db.Alts) do
									if a.Order == value then
										a.Order = alt.Order
									end
								end
								alt.Order = value
								redrawoptions()
							end,
						}
						altoptions[altname.."_show"] = {
							order = (alt.Order * 4) - 1,
							name = "",
							type = "select",
							width = "half",
							values = {["Yes"] = "Show",["No"] = "Hide"},
							get = function(info) return alt.Show end,
							set = function(info, value) alt.Show = value end,
						}
						altoptions[altname.."_del"] = {
							order = (alt.Order * 4),
							name = DELETE,
							desc = DELETE.." "..altname,
							type = "execute",
							width = "half",
							func = function(frame, button)
									if button == "LeftButton" then
										if altname == thisAlt then
											message("Cannot delete the current character, try full reset.")
											return
										else 
											RemoveAlt(altname)
										end
									end
								end,
						}
					end
					return altoptions
				end)()
			}
			ntoptions["Custom"] = {
				order = 4,
				type = "group",
				name = "Customise",
				args = (function()
					local customtable = {}
					local ordernum = 1
					customtable["option_currencies"] = {
						order = ordernum,
						type = "description",
						name = "Currency",
					}
					ordernum = ordernum + 1
					for n, c in pairs(addon.db.Currency) do
						local cname = C_CurrencyInfo.GetCurrencyInfo(c.ID).name
						customtable["currency "..cname] = {
							order = ordernum,
							type = "toggle",
							name = cname,
							get = function(info) return c.Show end,
							set = function(info, value) addon.db.Currency[n].Show = value; redrawoptions() end,
						}
						ordernum = ordernum + 1
					end
					customtable["option_quests"] = {
								order = ordernum,
								type = "description",
								name = "Quests",
					}
					ordernum = ordernum + 1
					customtable["option_weekly"] = {
								order = ordernum,
								type = "toggle",
								name = "Auto-Accept Weekly Bonus Quest",
								get = function(info) return addon.db.Tooltip.AcceptWeekly end,
								set = function(info, value) addon.db.Tooltip.AcceptWeekly = value; redrawoptions() end,
					}
					ordernum = ordernum + 1
					return customtable
				end)()
			}
			ntoptions["Reset"] = {
				order = 5,
				type = "group",
				name = "Reset",
				args = (function()
					local resettable = {}
					resettable["resetbutton"] = {
						type = "execute",
						name = "Reset Database",
						desc = "CAUTION: Clears all data, starts a new database",
						func = function(frame, button)
							if button == "LeftButton" then
								message("Database Reset")
								resetDB()
							end
						end,
					}
					return resettable
				end)()
			}
			ntoptions["Commands"] = {
				order = 6,
				type = "description",
				name = " \n Type '/nott' or '/nottoxic' to show the addon window. \n \n Typing 'config' or 'minimap' after that opens these options, and toggles the minimap icon. \n",
			}
			return ntoptions
		end)(),
	}
	setoptionstable = setoptionstable or {}
	for a, b in pairs(optionstable) do
		setoptionstable[a] = b
	end
end

function core:SetupOptions()
	core:MakeOptions()
	LibStub('AceConfigRegistry-3.0'):RegisterOptionsTable("Not Toxic", setoptionstable)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Not Toxic", nil, nil)
end

function core:OnEnable()
	self:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN", function() addon:UpdateCharData() end)
	self:RegisterEvent("PLAYER_ENTERING_WORLD", function()
		addon:UpdateCharData();
		addon:UpdateCovenantData();
	end)
	self:RegisterEvent("PLAYER_ALIVE", function()
		C_Timer.After(30, function()
			addon:UpdateCallingData()
			if not addon.db.Alts[thisAlt].WeeklyQuestOffer and addon.db.Alts[thisAlt].Level == MaxLevel and addon.db.Tooltip.AcceptWeekly then
				addon:PickupWeekly()
			end
		end)
	end)
	self:RegisterEvent("PLAYER_LOGOUT", function() addon:UpdateCharData() end)
	self:RegisterEvent("MYTHIC_PLUS_NEW_WEEKLY_RECORD", function() addon:UpdateMythicData() end)
	self:RegisterEvent("MYTHIC_PLUS_CURRENT_AFFIX_UPDATE", function() addon:UpdateMythicData() end)
	self:RegisterEvent("QUEST_LOG_UPDATE", function() C_Timer.After(1,function() addon:UpdateCallingData() end) end)
	self:RegisterBucketEvent("UPDATE_FACTION", 0.25, function() addon:UpdateFactionData() end) -- rep tracking
	self:RegisterBucketEvent("CURRENCY_DISPLAY_UPDATE", 0.25, function() addon:UpdateCurrency() end) -- currency tracking
	self:RegisterBucketEvent("BAG_UPDATE_DELAYED", 0.25, function() addon:UpdateItems() end) -- item tracking
	
	self:RegisterEvent("SOULBIND_ACTIVATED", function() addon:UpdateCovenantData() end)
	self:RegisterEvent("SOULBIND_FORGE_INTERACTION_ENDED", function() addon:UpdateCovenantData() end)
	self:RegisterEvent("COVENANT_CHOSEN", function() addon:UpdateCovenantData() end)
	self:RegisterEvent("COVENANT_SANCTUM_RENOWN_LEVEL_CHANGED", function() addon:UpdateCovenantData() end)
	self:RegisterEvent("WEEKLY_REWARDS_UPDATE", function() addon:UpdateRewardData() end)
end

function core:OnDisable()
	self:UnregisterAllEvents()
end

function core:ShowTooltip(aframe)
	if tip then qtip:Release(tip); tip = nil end
	tip = qtip:Acquire("NotToxicTooltip", 1, "LEFT")
	tip:SetCellMarginH(0)
	tip:SetCellMarginV(0)
	if addon.IsDetached() then
		tip:SetPoint("CENTER", aframe, "CENTER")
	else
		tip:SmartAnchorTo(aframe)
	end
	tip:Clear()
	tip:SetScale(addon.db.Tooltip.Scale)
	tip:SetFrameLevel(101)
	local thisLine = 0
	thisLine = AddSpace()
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, GOLDFONT.."Not Toxic"..FONTEND)
	for _, alt in pairs(addon.db.Alts) do
		if alt.Show == "Yes" then
			alt.Column = (alt.Order + 1)
		else
			alt.Column = 0
		end
		tip:AddColumn("CENTER")
	end
	
	for n, alt in pairs(addon.db.Alts) do
		if alt.Show == "Yes" then
			local altstring = string.gsub(n, '%s.+', "")
			if n == thisAlt then
				altstring = "<"..altstring..">"
				tip:SetColumnColor(alt.Column, RAID_CLASS_COLORS[alt.Class].r, RAID_CLASS_COLORS[alt.Class].g, RAID_CLASS_COLORS[alt.Class].b, 0.2)
			end
			tip:SetCell(thisLine, alt.Column, ClassColor(alt.Class, altstring))
			if alt.ItemLevel then
				tip:SetCellScript(thisLine, alt.Column, "OnEnter", function(frame) addon:showchardata(frame, alt) end)
				tip:SetCellScript(thisLine, alt.Column, "OnLeave", function(frame) if minitip then qtip:Release(minitip); minitip = nil end end)
			end
		end
	end
	
	thisLine = AddSpace()
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, YELLOWFONT.."Covenant"..FONTEND)
	for _, alt in pairs(addon.db.Alts) do
		if alt.Covenant and alt.Show == "Yes" then
			local ctext = alt.Covenant.name
			if alt.Covenant.ID ~= 0 then
				ctext = ctext .. " (" .. tostring(alt.Renown) .. ")"
			end
			tip:SetCell(thisLine, alt.Column, ClassColor(alt.Class, ctext))
			if alt.Covenant.ID ~= 0 then
				tip:SetCellScript(thisLine, alt.Column, "OnEnter", function(frame) addon:CovenantHoverFrame(frame, alt) end)
				tip:SetCellScript(thisLine, alt.Column, "OnLeave", function(frame) if covtip then qtip:Release(covtip); covtip = nil end end)
			end
		end
	end
	
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, YELLOWFONT.."Soulbind"..FONTEND)
	for _, alt in pairs(addon.db.Alts) do
		if alt.Soulbind and alt.Show == "Yes" then
			tip:SetCell(thisLine, alt.Column, ClassColor(alt.Class, alt.Soulbind.name))
		end
	end
	
	
	
	addon:UpdateMythicData()
	thisLine = AddSpace()
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, GOLDFONT.."Current Key"..FONTEND)
	for _, alt in pairs(addon.db.Alts) do
		if alt.CurrentKeyID and alt.Level == MaxLevel and alt.Show == "Yes" then
			tip:SetCell(thisLine, alt.Column, GOLDFONT..alt.CurrentKeyID..FONTEND..YELLOWFONT.." ("..alt.CurrentKeyLevel..")"..FONTEND)
			if alt.MythicRunHistory then
				tip:SetCellScript(thisLine, alt.Column, "OnEnter", function(frame) addon:MythicHoverFrame(frame, alt) end)
				tip:SetCellScript(thisLine, alt.Column, "OnLeave", function(frame) if mythictip then qtip:Release(mythictip); mythictip = nil end end)
			end
		end
	end
	
	addon:UpdateRewardData()
	thisLine = AddSpace()
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, GOLDFONT.."Raid Bosses"..FONTEND)
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, GOLDFONT.."Mythic+"..FONTEND)
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, GOLDFONT.."Honor"..FONTEND)
	for _, alt in pairs(addon.db.Alts) do
		if alt.Level == MaxLevel and alt.RaidChest and alt.Show == "Yes" then
			local raidprog = ""
			local endraidprog = false
			for i = 1,3 do
				if alt.RaidChest[i].level > 0 then
					raidprog = raidprog..GREENFONT.."\124T" .. READY_CHECK_READY_TEXTURE .. ":0|t "..FONTEND
				else
					if not endraidprog then
						raidprog = raidprog..YELLOWFONT..alt.RaidChest[i].progress.."/"..alt.RaidChest[i].threshold..FONTEND
						endraidprog = true
					end
				end
			end
			tip:SetCell(thisLine-2, alt.Column, raidprog)
			tip:SetCellScript(thisLine-2, alt.Column, "OnMouseUp", function(frame, button) LoadAddOn("Blizzard_WeeklyRewards"); if WeeklyRewardsFrame:IsShown() then WeeklyRewardsFrame:Hide() else WeeklyRewardsFrame:Show() end end)
		end
		if alt.Level == MaxLevel and alt.MythicChest and alt.Show == "Yes" then
			local mythicprog = ""
			local endmythicprog = false
			for i = 1,3 do
				if alt.MythicChest[i].level > 0 then
					mythicprog = mythicprog..GREENFONT.."\124T" .. READY_CHECK_READY_TEXTURE .. ":0|t "..FONTEND
				else
					if not endmythicprog then
						mythicprog = mythicprog..YELLOWFONT..alt.MythicChest[i].progress.."/"..alt.MythicChest[i].threshold..FONTEND
						endmythicprog = true
					end
				end
			end
			tip:SetCell(thisLine-1, alt.Column, mythicprog)
			tip:SetCellScript(thisLine-1, alt.Column, "OnMouseUp", function(frame, button) LoadAddOn("Blizzard_WeeklyRewards"); if WeeklyRewardsFrame:IsShown() then WeeklyRewardsFrame:Hide() else WeeklyRewardsFrame:Show() end end)
		end
		if alt.Level == MaxLevel and alt.PvpChest and alt.Show == "Yes" then
			local pvpprog = ""
			local endpvpprog = false
			for i = 1,3 do
				if alt.PvpChest[i].level > 0 then
					pvpprog = pvpprog..GREENFONT.."\124T" .. READY_CHECK_READY_TEXTURE .. ":0|t "..FONTEND
				else
					if not endpvpprog then
						pvpprog = pvpprog..YELLOWFONT..alt.PvpChest[i].progress.."/"..alt.PvpChest[i].threshold..FONTEND
						endpvpprog = true
					end
				end
			end
			tip:SetCell(thisLine, alt.Column, pvpprog)
			tip:SetCellScript(thisLine, alt.Column, "OnMouseUp", function(frame, button) LoadAddOn("Blizzard_WeeklyRewards"); if WeeklyRewardsFrame:IsShown() then WeeklyRewardsFrame:Hide() else WeeklyRewardsFrame:Show() end end)
		end
	end
	local showcollect = false
	for _, alt in pairs(addon.db.Alts) do
		if alt.CollectChest and alt.Show == "Yes" then
			if not showcollect then
				thisLine = AddLine(thisLine)
				tip:SetCell(thisLine, 1, GOLDFONT.."Loot To Collect"..FONTEND)
				showcollect = true
			end
			tip:SetCell(thisLine, alt.Column, "\124T"..READY_CHECK_READY_TEXTURE..":0|t")
		end
	end
	
	
	addon:UpdateTorghast()
	thisLine = AddSpace()
	local wingsknown = false
	local wings = {}
	thisLine = AddLine(thisLine)
	thisLine = AddLine(thisLine)
	for _, alt in pairs(addon.db.Alts) do
		if alt.Torghast then
			for wing, level in pairs(alt.Torghast) do
				if not wings[1] then
					wings[1] = wing
					tip:SetCell(thisLine-1, 1, YELLOWFONT..wing..FONTEND)
				else
					if wings[1] ~= wing and not wings[2] then
						wings[2] = wing
						tip:SetCell(thisLine, 1, YELLOWFONT..wing..FONTEND)
					end
				end
			end
		end
	end
	for _,alt in pairs(addon.db.Alts) do
		if alt.Torghast and alt.Show == "Yes" then
			if alt.Torghast[wings[1]] then tip:SetCell(thisLine-1, alt.Column, ClassColor(alt.Class, "Layer "..alt.Torghast[wings[1]])) end
			if alt.Torghast[wings[2]] then tip:SetCell(thisLine, alt.Column, ClassColor(alt.Class, "Layer "..alt.Torghast[wings[2]])) end
		end
	end
	
	local showcallings = false
	for _,alt in pairs(addon.db.Alts) do
		if alt.Callings and alt.Show == "Yes" then
			showcallings = true
		end
	end
	
	if showcallings then
		thisLine = AddSpace()
		thisLine = AddLine(thisLine)
		tip:SetCell(thisLine, 1, GOLDFONT.."Callings"..FONTEND)
		for day = 1,3 do
			thisLine = AddLine(thisLine)
			local grammar = ""
			if day == 2 then grammar = "1 Day, " end
			if day == 3 then grammar = "2 Days, " end
			tip:SetCell(thisLine, 1, YELLOWFONT..grammar..tostring(floor(GetQuestResetTime()/3600)).." Hours"..FONTEND)
			for _,alt in pairs(addon.db.Alts) do
				if alt.Callings and alt.Show == "Yes" then
					if alt.Callings[day].ID <= 0 then
						if alt.Callings[day].ID < 0 then
							tip:SetCell(thisLine, alt.Column, GRAYFONT..""..FONTEND)
						else
							tip:SetCell(thisLine, alt.Column, REDFONT.."Pickup Quest"..FONTEND)
						end
					else
						if not alt.Callings[day].C then
							tip:SetCell(thisLine, alt.Column, YELLOWFONT.."Active"..FONTEND)
							if alt.Callings[day].QL then
								tip:SetCellScript(thisLine, alt.Column, "OnEnter", function(frame) addon:showlink(frame, alt.Callings[day].QL) end)
								tip:SetCellScript(thisLine, alt.Column, "OnLeave", function(frame) GameTooltip:Hide() end)
							end
						else
							tip:SetCell(thisLine, alt.Column, GREENFONT.."Complete"..FONTEND)
						end
					end
				end
			end
		end
	end
	
	thisLine = AddSpace()
	thisLine = AddLine(thisLine)
	for n, alt in pairs(addon.db.Alts) do
		if alt.WeeklyQuest and alt.WeeklyMin and alt.WeeklyMax and alt.Show == "Yes" then 
			tip:SetCell(thisLine, 1, GetQuestLink(alt.WeeklyQuest))
			tip:SetCellScript(thisLine, 1, "OnEnter", function(frame) addon:showlink(frame, GetQuestLink(alt.WeeklyQuest)) end)
			tip:SetCellScript(thisLine, 1, "OnLeave", function(frame) GameTooltip:Hide() end)
			local currentprogress = alt.WeeklyMin.."/"..alt.WeeklyMax
			tip:SetCell(thisLine, alt.Column, GOLDFONT..currentprogress..FONTEND)
		else
			if alt.WeeklyQuest and C_QuestLog.IsQuestFlaggedCompleted(alt.WeeklyQuest) and alt.Show == "Yes" then
				tip:SetCell(thisLine, alt.Column, GOLDFONT.."Weekly\nComplete"..FONTEND)
			else
				if n == thisAlt and alt.Show == "Yes" then
					tip:SetCell(thisLine, alt.Column, GOLDFONT.."Get Weekly"..FONTEND)
					tip:SetCellScript(thisLine, alt.Column, "OnMouseUp", function(frame, button) addon:PickupWeekly() end)
				end
			end
		end
	end
	
	thisLine = AddLine(thisLine)
	for n, alt in pairs(addon.db.Alts) do
		if alt.Level >= MaxLevel and alt.Show == "Yes" then
			tip:SetCell(thisLine, 1, GOLDFONT.."World Boss"..FONTEND)
			if alt.WeeklyBoss then
				tip:SetCell(thisLine, alt.Column, GREENFONT.."Killed"..FONTEND)
			else
				tip:SetCell(thisLine, alt.Column, REDFONT.."Not Killed"..FONTEND)
			end
		end
	end
	
	
	addon:UpdateFactionData()
	thisLine = AddSpace()
	for n, f in ipairs(addon.db.Factions) do
		if f.Show then
			local fshown = false
			local flabel = GetFactionInfoByID(f.ID)
			if flabel then
				for _, alt in pairs(addon.db.Alts) do
					if alt.Reputation and alt.Reputation[f.ID] and alt.Reputation[f.ID].Standing ~= "" and alt.Show == "Yes" then
						if not fshown then
							thisLine = AddLine(thisLine)
							tip:SetCell(thisLine,1, YELLOWFONT..flabel..FONTEND)
							fshown = true
						end
						local percentRep = ""
						if alt.Reputation[f.ID]["Percent"] then percentRep = alt.Reputation[f.ID]["Percent"].."% " end
						tip:SetCell(thisLine, alt.Column, RepColor(alt.Reputation[f.ID]["Standing"], percentRep .. alt.Reputation[f.ID]["Standing"]))
					end
				end
			end
		end
	end
	
	addon:UpdateCurrency()
	addon:UpdateItems()
	thisLine = AddSpace()
	for n, currency in ipairs(addon.db.Currency) do
		if currency.Show then
			local currencyshown = false
			for _, alt in pairs(addon.db.Alts) do
				if alt.Show == "Yes" and alt.Currency and alt.Currency[currency.ID] then
					if not currencyshown then
						local ct = C_CurrencyInfo.GetCurrencyInfo(currency.ID).name
						thisLine = AddLine(thisLine)
						tip:SetCell(thisLine, 1, "\124T"..alt.Currency[currency.ID]["Icon"]..":0\124t "..GOLDFONT..ct..FONTEND)
						tip:SetCellScript(thisLine, 1, "OnEnter", function(frame) addon:showlink(frame, C_CurrencyInfo.GetCurrencyLink(currency.ID, 0)) end)
						tip:SetCellScript(thisLine, 1, "OnLeave", function(frame) GameTooltip:Hide() end)
						currencyshown = true
					end
					if currency.ID == 1813 and alt.BagAnima and alt.BagAnima > 0 then
						tip:SetCell(thisLine, alt.Column, ClassColor(alt.Class, alt.Currency[currency.ID]["Total"].." ("..(alt.Currency[currency.ID]["Total"]+alt.BagAnima)..")"))
					else
						tip:SetCell(thisLine, alt.Column, ClassColor(alt.Class, alt.Currency[currency.ID]["Total"]))
					end
				end
			end
		end
	end
	
	
	
	
	for i=4,thisLine do
		tip:SetLineScript(i, "OnEnter", function() end)
		tip:SetLineScript(i, "OnLeave", function() end)
	end
	
	thisLine = AddSpace()
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, GRAYFONT.."Left Click - Detach"..FONTEND)
	thisLine = AddLine(thisLine)
	tip:SetCell(thisLine, 1, GRAYFONT.."Right Click - Options"..FONTEND)
	
	addon:SkinThis(tip)
	if not addon.IsDetached() then
		tip:SetAutoHideDelay(0.1, aframe)
	end
	tip:Show()
end

function AddSpace()
	return tip:AddSeparator(3,0,0,0,0)
end

function AddLine(thisLine)
	tip:AddLine()
	thisLine = thisLine + 1
	return thisLine
end

function RemoveAlt(altname)
	for _, alt in pairs(addon.db.Alts) do
		if addon.db.Alts[altname].Order < alt.Order then
			alt.Order = alt.Order - 1
		end
	end
	addon.db.Alts[altname] = nil
	redrawoptions()
	message(altname.." was deleted")
end

function resetDB()
	addon.db = nil
	NotToxicDB = nil
	NotToxicDB = addon.defaultDB
	addon.db = NotToxicDB
	
	core:CharacterInit()
	addon:UpdateCharData()
	redrawoptions()
end

function redrawoptions()
	core:MakeOptions()
	if _G.InterfaceOptionsFrame:IsShown() then
		_G.InterfaceOptionsFrame:Hide();
		InterfaceOptionsFrame_OpenToCategory("Not Toxic")
		InterfaceOptionsFrame_OpenToCategory("Not Toxic")
	end
end
